import { useEffect, useState } from "react";
import {
  Button,
  Checkbox,
  Drawer,
  Form,
  Input,
  message,
  Modal,
  PaginationProps,
  Popconfirm,
  Space,
  Table,
  Timeline,
  Typography,
} from "antd";
import { Store } from "antd/lib/form/interface";
import { inject, observer } from "mobx-react";
import "./source.less";
import { DataType } from "@/util/model/interface";
import Upload from "./upload";
import toExcel from "@/util/xmsx";
import Share from "./share";
const Source = (props: Store) => {
  const { sourceStore } = props;
  const [form] = Form.useForm();
  const [coloums, setColumns] = useState<any>([]);
  const [content, setContent] = useState([]);
  const [selectKey, setSelectKey] = useState<Array<string>>([]);
  const [page, setPage] = useState<number>(1);
  const [editingKey, setEditingKey] = useState("");
  const [loading, setLoading] = useState(false);
  const [selectedRowKeys, setSelectedRowKeys] = useState<React.Key[]>([]);
  const [rowKeys, setRowKeys] = useState<any[]>([]);
  const isEditing = (record) => record.key === editingKey;
  const [open, setOpen] = useState<boolean>(false);
  const [shareOpen, setShareOpen] = useState<boolean>(false);

  const edit = (record: any) => {
    form.setFieldsValue({ ...record });
    setEditingKey(record.key);
  };
  // 获取列表数据
  useEffect(() => {
    sourceStore.getHead().then((res) => {
      res.forEach((element) => {
        element.dataIndex = element.identity;
        element.title = element.data_name;
        element.label = element.data_name;
        element.value = element.identity;
        element.editable = true;
      });
      setColumns(res);
    });
    getContent([], 1);
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [sourceStore]);
  const save = async (key: React.Key) => {
    try {
      const row = (await form.validateFields()) as DataType;
      const newData: any = [...content];
      const index = newData.findIndex((item: any) => key === item.key);
      if (index > -1) {
        const item = newData[index];
        setLoading(true);
        let res = await sourceStore.modefyData(row, item.id_card);
        if (res) {
          getContent(selectKey, 1);
        }
        setLoading(false);
        setEditingKey("");
      }
    } catch (errInfo) {
      console.log("Validate Failed:", errInfo);
    }
  };
  const getContent = (list, index) => {
    setLoading(true);
    sourceStore.geContent(list, index, 20).then((res) => {
      res.forEach((element) => {
        element.key = element.identity;
      });
      setContent(res);
      setLoading(false);
    });
  };
  const getHead = () => {
    let head = coloums.filter((value) => selectKey.includes(value.identity));
    return head;
  };

  const saveClo = () => {
    let list: any = [];
    if (rowKeys.length === 0) {
      message.info("请选择需要导出的列");
      return;
    }
    for (let i = 0; i < rowKeys.length; i++) {
      let obj = {};
      for (const key in rowKeys[i]) {
        if (key.indexOf("dbs") > -1) {
          obj[key] = rowKeys[i][key];
        }
      }
      list.push(obj);
    }
    toExcel(getHead(), list, "test.xlsx", "Sheet1");
  };
  const cancel = () => {
    setEditingKey("");
  };
  interface EditableCellProps extends React.HTMLAttributes<HTMLElement> {
    editing: boolean;
    dataIndex: string;
    fixed: string;
    index: number;
  }
  const actionCloumn = {
    title: "操作",
    with: 200,
    fixed: "right",
    render: (_: any, record) => {
      const editable = isEditing(record);
      return editable ? (
        <span>
          <Typography.Link
            onClick={() => {
              save(record.key);
            }}
            style={{ marginInlineEnd: 8 }}
          >
            保存
          </Typography.Link>
          <Popconfirm title="Sure to cancel?" onConfirm={cancel}>
            取消
          </Popconfirm>
        </span>
      ) : (
        <Space>
          <Typography.Link
            disabled={editingKey !== ""}
            onClick={() => edit(record)}
          >
            编辑
          </Typography.Link>
          <Typography.Link onClick={() => setOpen(true)}>
            查看事件
          </Typography.Link>
          <Typography.Link onClick={() => setShareOpen(true)}>
            分享
          </Typography.Link>
        </Space>
      );
    },
  };

  const EditableCell: React.FC<React.PropsWithChildren<EditableCellProps>> = ({
    editing,
    dataIndex,
    children,
    ...restProps
  }) => {
    const inputNode = <Input />;

    return (
      <td {...restProps}>
        {editing ? (
          <Form.Item name={dataIndex} style={{ margin: 0 }}>
            {inputNode}
          </Form.Item>
        ) : (
          children
        )}
      </td>
    );
  };

  const mergedColumns = coloums.map((col) => {
    if (!col.editable) {
      return col;
    }
    return {
      ...col,
      onCell: (record: DataType) => ({
        record,
        dataIndex: col.dataIndex,
        title: col.title,
        editing: isEditing(record),
      }),
    };
  });

  const onChange: PaginationProps["onChange"] = (page) => {
    setPage(page);
    console.log(page)
    getContent(selectKey, page);
    cancel();
  };
  const rowSelection = {
    selectedRowKeys,
    preserveSelectedRowKeys: true,
    fixed: true,
    onChange: (keys, rowKeys, info) => {
      setSelectedRowKeys(keys);
      setRowKeys(rowKeys);
    },
  };
  return (
    <div className="contentBox">
      <Space direction="vertical" size="middle" style={{ display: "flex" }}>
        <Upload />
        <Space>
          <Checkbox.Group
            options={coloums}
            defaultValue={selectKey}
            onChange={(v) => {
              setSelectKey(v);
              getContent(v, 1);
            }}
          />
          <Button onClick={saveClo}>导出</Button>
        </Space>

        <Form form={form} component={false}>
          <Table
            loading={loading}
            scroll={{ x: "max-content", scrollToFirstRowOnChange: true }}
            components={{
              body: { cell: EditableCell },
            }}
            bordered
            rowSelection={rowSelection}
            dataSource={content}
            columns={[...mergedColumns, actionCloumn]}
            rowClassName="editable-row"
            pagination={{
              onChange: onChange,
              total: sourceStore.total,
              current: page,
              pageSize: 20,
              showSizeChanger: false,
            }}
          />
        </Form>
      </Space>

      <Drawer
        closable
        title="事件"
        placement="right"
        open={open}
        loading={loading}
        onClose={() => setOpen(false)}
      >
        <Timeline
          items={[
            {
              children: <div>
                <p>事件名称： 防洪治理</p>
                <p>事件角色： 负责人</p>
                <p>事件影响： 为解决洪水灾害，兄弟们大胆往前冲</p>
                <p>事件处理： 描述</p>
                
                <p>处理结果：搞得好</p>
                <p>事件描述： 为解决洪水灾害，兄弟们大胆往前冲</p>
                <p>事件地点： 为解决洪水灾害，兄弟们大胆往前冲</p>
                 <p>事件开始时间： 2025-02-12</p>
              </div>,
            },
             {
              children: <div>
                <p>事件名称： 民众安抚</p>
                <p>事件角色： 负责人</p>
                <p>事件影响： 影响很大哟</p>
                <p>事件处理： 描述</p>
                <p>处理结果：搞得好</p>
                <p>事件描述： 为解决洪水灾害，兄弟们大胆往前冲</p>
                <p>事件地点： 为解决洪水灾害，兄弟们大胆往前冲</p>
                              <p>事件结束时间： 2025-03-12</p>
              </div>,
            },
             {
              children: <div>
                <p>事件名称： 民众安抚</p>
                <p>事件角色： 负责人</p>
                <p>事件影响： 影响很大哟</p>
                <p>事件处理： 描述</p>
                <p>处理结果：搞得好</p>
                <p>事件描述： 为解决洪水灾害，兄弟们大胆往前冲</p>
                <p>事件地点： 为解决洪水灾害，兄弟们大胆往前冲</p>
                <p>事件开始时间： 2025-02-12</p>
                <p>事件结束时间： 2025-03-12</p>
              </div>,
            },
             {
              children: <div>
                <p>事件名称： 民众安抚</p>
                <p>事件角色： 负责人</p>
                <p>事件影响： 影响很大哟</p>
                <p>事件处理： 描述</p>
                <p>处理结果：搞得好</p>
                <p>事件描述： 为解决洪水灾害，兄弟们大胆往前冲</p>
                <p>事件地点： 为解决洪水灾害，兄弟们大胆往前冲</p>
                  <p>事件开始时间： 2025-02-12</p>
                <p>事件结束时间： 2025-03-12</p>
              </div>,
            },
          ]}
        />
      </Drawer>
      <Modal
        open={shareOpen}
        title="分享"
        onOk={() => { }}
        onCancel={() => { setShareOpen(false) }}
      >
        <Share />
      </Modal>
    </div>
  );
};

export default inject("sourceStore")(observer(Source));
